﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using Webus.Index;
using Webus.IO;
using Webus.Documents;
using Webus.Helpers;
using Webus.Search;
using System.Threading.Tasks;
using Webus.Search.Highlight;

namespace CodeSearch
{
    public partial class frmCodeSearch : Form
    {
        public frmCodeSearch()
        {
            InitializeComponent();
        }

        private void btnAbout_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.gdtsearch.com");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ExceptionHelper.ToString(ex));
            }
        }

        IIndexer m_Index;
        ISearcher m_Searcher;

        private void btnBuildIndex_Click(object sender, EventArgs e)
        {
            try
            {
                if (folderBrowserDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    Task.Factory.StartNew(IndexProc);
                    txtKeyword.Enabled = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ExceptionHelper.ToString(ex));
            }
        }

        public delegate void StatusChangeEventHandler(object sender, string status);
        public event StatusChangeEventHandler StatusChanged;

        void IndexProc()
        {
            var files = Directory.GetFiles(folderBrowserDlg.SelectedPath, "*.cs", SearchOption.AllDirectories);
            if (files != null && files.Length > 0)
            {
                if (StatusChanged != null)
                {
                    this.StatusChanged(this, string.Format("0/{0}", files.Length));
                }
                this.BuildIndex();
                foreach (var file in files)
                {
                    Document doc = new Document();
                    doc.Fields.Add(new Field("FileName", file, FieldAttributes.None));
                    doc.Fields.Add(new Field("Code", StringHelper.LoadString(file), FieldAttributes.AnalyseIndex | FieldAttributes.UnStore));
                    m_Index.Add(doc);
                    if (StatusChanged != null)
                    {
                        this.StatusChanged(this, string.Format("{0}/{1}: {2}", m_Index.DocCount, files.Length, file));
                    }
                }
            }
            if (StatusChanged != null)
            {
                this.StatusChanged(this, string.Format("{0}/{1}: Ready!", m_Index.DocCount, files.Length));
            }
        }

        void BuildIndex()
        {
            if (m_Index != null)
            {
                m_Index.Close();
            }
            m_Index = new IndexManager(new CodeAnalyzer());
            m_Index.DumpDocs = 3000;
            m_Index.DumpSize = 10;
            m_Index.MinIndexSize = int.MaxValue;
            m_Index.MaxIndexSize = int.MaxValue;
            m_Index.MergeFactor = int.MaxValue;
            m_Index.New(folderBrowserDlg.SelectedPath + @"\CodeSearch.Index");
            m_Searcher = new IndexSearcher(m_Index);
        }

        private void txtKeyword_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void frmCodeSearch_Load(object sender, EventArgs e)
        {
            try
            {
                this.StatusChanged += new StatusChangeEventHandler(frmCodeSearch_StatusChanged);
                Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(txtSourceCode, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.CSharp);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ExceptionHelper.ToString(ex));
            }
        }

        delegate void UpdateUI();
        void frmCodeSearch_StatusChanged(object sender, string status)
        {
            if (status.Length > 100)
            {
                status = status.Substring(0, 50) + "..." + status.Substring(status.Length - 30, 30);
            }
            this.Invoke(new UpdateUI(() => { this.txtStatus.Text = status; }));
        }

        private void dgvResult_SelectionChanged(object sender, EventArgs e)
        {
            try
            {
                if (dgvResult.SelectedRows != null && dgvResult.SelectedRows.Count > 0)
                {
                    SearchResult sr = (dgvResult.DataSource as List<SearchResult>)[dgvResult.SelectedRows[0].Index];
                    var code = sr.Details;
                    txtSourceCode.Document.Text = code;
                    btnHighlight.Text = "Hints: " + sr.Hints;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ExceptionHelper.ToString(ex));
            }
        }

        private void btnHighlight_Click(object sender, EventArgs e)
        {
            try
            {
                txtSourceCode.FindNext(txtKeyword.Text, false, false, false);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ExceptionHelper.ToString(ex));
            }
        }

        private void btnOpenIndex_Click(object sender, EventArgs e)
        {
            if (folderBrowserDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if (m_Index != null)
                {
                    m_Index.Close();
                }
                m_Index = new IndexManager(new CodeAnalyzer());
                m_Index.Open(folderBrowserDlg.SelectedPath, IndexOpenMode.Read);
                m_Searcher = new IndexSearcher(m_Index);
                txtKeyword.Enabled = true;
            }
        }

        private void frmCodeSearch_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (m_Index != null)
            {
                m_Index.Close();
            }
        }

        private void txtKeyword_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                try
                {
                    var hits = m_Searcher.Search(string.Format(txtQueryExpr.Text, txtKeyword.Text.ToLower()));
                    List<SearchResult> result = new List<SearchResult>();
                    foreach (HitDoc hit in hits)
                    {
                        StandardHighlighter hl = new StandardHighlighter(hit);
                        result.Add(new SearchResult(hit));
                    }
                    dgvResult.DataSource = result;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ExceptionHelper.ToString(ex));
                }
            }
        }

    }

    class SearchResult
    {
        HitDoc m_HitDoc;
        public SearchResult(HitDoc hit)
        {
            m_HitDoc = hit;
        }
        public string FileName
        {
            get
            {
                return m_HitDoc.GetDoc().GetField("FileName").Value.ToString();
            }
        }
        public int Hints
        {
            get
            {
                return m_HitDoc.Positions.Count;
            }
        }
        public string Details
        {
            get
            {
                var code = StringHelper.LoadString(this.FileName);
                //if (code.Length > 100000)
                //{
                //    code = code.Substring(0, 100000);
                //}
                return code;
            }
        }
    }
}
